@@ -24,7 +24,6 @@ import java.awt.datatransfer.DataFlavor;
 import java.awt.datatransfer.Transferable;
 import java.awt.datatransfer.UnsupportedFlavorException;
 import java.awt.dnd.DnDConstants;
-import java.awt.image.BufferedImage;
 import java.io.File;
 import java.io.IOException;
 import java.io.StringReader;
@@ -46,7 +45,6 @@ import javax.swing.text.ElementIterator;
 import javax.swing.text.html.HTMLDocument;
 import javax.swing.text.html.HTMLEditorKit;
 
-import org.apache.commons.lang.StringUtils;
 import org.freeplane.core.resources.ResourceController;
 import org.freeplane.core.ui.ExampleFileFilter;
 import org.freeplane.core.ui.components.UITools;
@@ -88,25 +86,16 @@ import org.freeplane.view.swing.features.filepreview.ViewerController.PasteMode;
 public class MClipboardController extends ClipboardController {
 	public static final String RESOURCES_REMIND_USE_RICH_TEXT_IN_NEW_NODES = "remind_use_rich_text_in_new_nodes";
 	private class DirectHtmlFlavorHandler implements IDataFlavorHandler {
-		private String textFromClipboard;
+		private final String textFromClipboard;
 
 		public DirectHtmlFlavorHandler(final String textFromClipboard) {
 			this.textFromClipboard = textFromClipboard;
 		}
 
 		void paste(final NodeModel target) {
-			textFromClipboard = cleanHtml(textFromClipboard);
-			final NodeModel node = Controller.getCurrentModeController().getMapController().newNode(textFromClipboard,
+			final String text = cleanHtml(textFromClipboard);
+			final NodeModel node = Controller.getCurrentModeController().getMapController().newNode(text,
 					Controller.getCurrentController().getMap());
-			final String text = textFromClipboard;
-			final Matcher m = HREF_PATTERN.matcher(text);
-			if (m.matches()) {
-				final String body = m.group(2);
-				if (!body.matches(".*<\\s*a.*")) {
-					final String href = m.group(1);
-					((MLinkController) LinkController.getController()).setLinkTypeDependantLink(node, href);
-				}
-			}
 			((MMapController) Controller.getCurrentModeController().getMapController()).insertNode(node, target);
 		}
 
@@ -124,21 +113,35 @@ public class MClipboardController extends ClipboardController {
 		}
 
 		public void paste(Transferable t, final NodeModel target, final boolean asSibling, final boolean isLeft, int dropAction) {
-	        if (LinkController.getLinkType() == LinkController.LINK_RELATIVE_TO_MINDMAP && target.getMap().getFile() == null) {
+			boolean copyFile = dropAction == DnDConstants.ACTION_COPY;
+	        final File mapFile = target.getMap().getFile();
+			if ((copyFile || LinkController.getLinkType() == LinkController.LINK_RELATIVE_TO_MINDMAP) && mapFile == null) {
 	        	JOptionPane.showMessageDialog(Controller.getCurrentController().getViewController().getCurrentRootComponent(),
 	        	    TextUtils.getText("map_not_saved"), "Freeplane", JOptionPane.WARNING_MESSAGE);
 	        	return;
 	        }
-			boolean pasteImages = dropAction == DnDConstants.ACTION_COPY;
 			ViewerController viewerController = ((ViewerController)Controller.getCurrentModeController().getExtension(ViewerController.class));
-			for (final File file : fileList) {
-				if(pasteImages  && viewerController.paste(file, target, PasteMode.valueOf(asSibling), isLeft)){
-					continue;
+			boolean pasteImagesFromFiles = ResourceController.getResourceController().getBooleanProperty("pasteImagesFromFiles");
+			for (final File sourceFile : fileList) {
+				final File file;
+				if(copyFile){
+					try {
+						file = new TargetFileCreator().createTargetFile(mapFile, sourceFile.getName());
+						file.getParentFile().mkdirs();
+						FileUtils.copyFile(sourceFile, file);
+					} catch (IOException e) {
+						LogUtils.warn(e);
+						continue;
+					}
+				}
+				else
+					file = sourceFile;
+				if(! pasteImagesFromFiles || !viewerController.paste(file, target, PasteMode.valueOf(asSibling), isLeft)) {
+					final MMapController mapController = (MMapController) Controller.getCurrentModeController().getMapController();
+					final NodeModel node = mapController.newNode(file.getName(), target.getMap());
+					((MLinkController) LinkController.getController()).setLinkTypeDependantLink(node, file);
+					mapController.insertNode(node, target, asSibling, isLeft, isLeft);
 				}
-				final MMapController mapController = (MMapController) Controller.getCurrentModeController().getMapController();
-				final NodeModel node = mapController.newNode(file.getName(), target.getMap());
-				((MLinkController) LinkController.getController()).setLinkTypeDependantLink(node, file);
-				mapController.insertNode(node, target, asSibling, isLeft, isLeft);
 			}
 		}
 	}
@@ -257,11 +260,11 @@ public class MClipboardController extends ClipboardController {
 			this.textFromClipboard = textFromClipboard;
 		}
 
-		private String addFragment(final HTMLDocument doc, final Element element, final int depth, final int start,
+		private void addFragment(final HTMLDocument doc, final Element element, final int depth, final int start,
 		                           final int end, final LinkedList<TextFragment> htmlFragments)
 		        throws BadLocationException, IOException {
 			final String paragraphText = doc.getText(start, end - start).trim();
-			if (paragraphText.length() > 0) {
+			if (paragraphText.length() > 0 || element.getName().equals("img")) {
 				final StringWriter out = new StringWriter();
 				new PasteHtmlWriter(out, element, doc, start, end - start).write();
 				final String string = out.toString();
@@ -271,7 +274,6 @@ public class MClipboardController extends ClipboardController {
 					htmlFragments.add(htmlFragment);
 				}
 			}
-			return paragraphText;
 		}
 
 		private Element getParentElement(final HTMLDocument doc) {
@@ -297,9 +299,9 @@ public class MClipboardController extends ClipboardController {
 
 		private void pasteHtmlWithoutRedisplay(final Object t, final NodeModel parent, final boolean asSibling,
 		                                       final boolean isLeft) {
-			String textFromClipboard = (String) t;
-			textFromClipboard = cleanHtml(textFromClipboard);
-			final TextFragment[] htmlFragments = split(textFromClipboard);
+			final String textFromClipboard = (String) t;
+			final String cleanedTextFromClipboard = cleanHtml(textFromClipboard);
+			final TextFragment[] htmlFragments = split(cleanedTextFromClipboard);
 			pasteStringWithoutRedisplay(htmlFragments, parent, asSibling, isLeft);
 		}
 
@@ -409,39 +411,35 @@ public class MClipboardController extends ClipboardController {
             	UITools.errorMessage(TextUtils.getRawText("map_not_saved"));
             	return;
             }
-            final String mmFileName = mindmapFile.getName();
-            String fileNameTemplate = mmFileName.substring(0, mmFileName.lastIndexOf('.')) + "_";
-            while (fileNameTemplate.length() < 3)
-            	fileNameTemplate = fileNameTemplate + '_';
             //file that we'll save to disk.
-            File file;
+            File imageFile;
             try {
-            	final File dir = mindmapFile.getParentFile();
-            	file = File.createTempFile(fileNameTemplate, "."+ImageAdder.IMAGE_FORMAT, dir);
-            	String imgfilepath=file.getAbsolutePath();
-            	File tempFile = file = new File(imgfilepath);
-            	final JFileChooser fileChooser = new JFileChooser(file);		
+            	imageFile = new TargetFileCreator().createTargetFile(mindmapFile, ImageAdder.IMAGE_FORMAT);
+    			imageFile.getParentFile().mkdirs();
+            	String imgfilepath=imageFile.getAbsolutePath();
+            	File tempFile = imageFile = new File(imgfilepath);
+            	final JFileChooser fileChooser = new JFileChooser(imageFile);		
             	final ExampleFileFilter filter = new ExampleFileFilter();
             	filter.addExtension(ImageAdder.IMAGE_FORMAT);
             	fileChooser.setAcceptAllFileFilterUsed(false);
             	fileChooser.setFileFilter(filter);
-            	fileChooser.setSelectedFile(file);
+            	fileChooser.setSelectedFile(imageFile);
             	int returnVal = fileChooser.showSaveDialog(UITools.getCurrentRootComponent());
             	if (returnVal != JFileChooser.APPROVE_OPTION) {
             		tempFile.delete();
             		return;
             	}
-            	file = fileChooser.getSelectedFile();
-            	if(tempFile.exists() && ! file.getAbsoluteFile().equals(tempFile)){
+            	imageFile = fileChooser.getSelectedFile();
+            	if(tempFile.exists() && ! imageFile.getAbsoluteFile().equals(tempFile)){
             		tempFile.delete();
             	}
-            	if(file.isDirectory())
+            	if(imageFile.isDirectory())
             		return;
-            	if(! FileUtils.getExtension(file.getName()).equals(ImageAdder.IMAGE_FORMAT))
-            		file = new File(file.getPath() + '.' + ImageAdder.IMAGE_FORMAT);
-            	final NodeModel node = mapController.newNode(file.getName(), target.getMap());
+            	if(! FileUtils.getExtension(imageFile.getName()).equals(ImageAdder.IMAGE_FORMAT))
+            		imageFile = new File(imageFile.getPath() + '.' + ImageAdder.IMAGE_FORMAT);
+            	final NodeModel node = mapController.newNode(imageFile.getName(), target.getMap());
             	mapController.insertNode(node, target, asSibling, isLeft, isLeft);
-            	new ImageAdder(image, mapController, mindmapFile, file).attachImageToNode(node);
+            	new ImageAdder(image, mapController, mindmapFile, imageFile).attachImageToNode(node);
             }
             catch (IOException e) {
             	e.printStackTrace();
@@ -450,8 +448,6 @@ public class MClipboardController extends ClipboardController {
 
     }
 	private static final Pattern HEADER_REGEX = Pattern.compile("h(\\d)", Pattern.CASE_INSENSITIVE);
-	private static final Pattern HREF_PATTERN = Pattern
-	    .compile("<html>\\s*<body>\\s*<a\\s+href=\"([^>]+)\">(.*)</a>\\s*</body>\\s*</html>");
 	private static final String RESOURCE_UNFOLD_ON_PASTE = "unfold_on_paste";
 	public static final String RESOURCES_CUT_NODES_WITHOUT_QUESTION = "cut_nodes_without_question";
 
@@ -472,17 +468,20 @@ public class MClipboardController extends ClipboardController {
 		createActions();
 	}
 
-	private String cleanHtml(String in) {
-		in = in.replaceFirst("(?i)(?s)<head>.*</head>", "").replaceFirst("(?i)(?s)^.*<html[^>]*>", "<html>")
+	private String cleanHtml(String content) {
+		content = content.replaceFirst("(?i)(?s)<head>.*</head>", "").replaceFirst("(?i)(?s)^.*<html[^>]*>", "<html>")
 		    .replaceFirst("(?i)(?s)<body [^>]*>", "<body>").replaceAll("(?i)(?s)<script.*?>.*?</script>", "")
 		    .replaceAll("(?i)(?s)</?tbody.*?>", "").replaceAll("(?i)(?s)<!--.*?-->", "").replaceAll(
 		        "(?i)(?s)</?o[^>]*>", "");
-		if (StringUtils.equals(ResourceController.getResourceController().getProperty(
-		    "cut_out_pictures_when_pasting_html"), "true")) {
-			in = in.replaceAll("(?i)(?s)<img[^>]*>", "");
+		if (ResourceController.getResourceController().getBooleanProperty("cut_out_pictures_when_pasting_html")) {
+			String contentWithoutImages = content.replaceAll("(?i)(?s)<img[^>]*>", "");
+			final boolean contentContainsOnlyImages = HtmlUtils.htmlToPlain(contentWithoutImages).trim().isEmpty();
+			if(! contentContainsOnlyImages) {
+				content = contentWithoutImages;
+			}
 		}
-		in = HtmlUtils.unescapeHTMLUnicodeEntity(in);
-		return in;
+		content = HtmlUtils.unescapeHTMLUnicodeEntity(content);
+		return content;
 	}
 
 	/**
@@ -535,17 +534,6 @@ public class MClipboardController extends ClipboardController {
 			catch (final IOException e) {
 			}
 		}
-		if (t.isDataFlavorSupported(MindMapNodesSelection.fileListFlavor)) {
-			try {
-				final List<File> fileList = castToFileList(t.getTransferData(MindMapNodesSelection.fileListFlavor));
-				if (!shouldIgnoreFileListFlavor(fileList))
-					return new FileListFlavorHandler(fileList);
-			}
-			catch (final UnsupportedFlavorException e) {
-			}
-			catch (final IOException e) {
-			}
-		}
 		final ResourceController resourceController = ResourceController.getResourceController();
 		DataFlavor supportedHtmlFlavor = getSupportedHtmlFlavor(t);
 		if (supportedHtmlFlavor != null) {
@@ -576,6 +564,17 @@ public class MClipboardController extends ClipboardController {
 			catch (final IOException e) {
 			}
 		}
+		if (t.isDataFlavorSupported(MindMapNodesSelection.fileListFlavor)) {
+			try {
+				final List<File> fileList = castToFileList(t.getTransferData(MindMapNodesSelection.fileListFlavor));
+				if (!shouldIgnoreFileListFlavor(fileList))
+					return new FileListFlavorHandler(fileList);
+			}
+			catch (final UnsupportedFlavorException e) {
+			}
+			catch (final IOException e) {
+			}
+		}
 		if (t.isDataFlavorSupported(DataFlavor.stringFlavor)) {
 			try {
 				final String plainTextFromClipboard = t.getTransferData(DataFlavor.stringFlavor).toString();
@@ -600,7 +599,7 @@ public class MClipboardController extends ClipboardController {
 	}
 
 	private boolean shouldIgnoreFileListFlavor(final List<File> fileList) {
-		if(fileList.isEmpty())
+		if(fileList == null || fileList.isEmpty())
 			return true;
 		final File file = fileList.get(0);
 		if(file.isDirectory())
@@ -696,10 +695,12 @@ public class MClipboardController extends ClipboardController {
 		if (t == null) {
 			return;
 		}
-		/*
-		 * DataFlavor[] fl = t.getTransferDataFlavors(); for (int i = 0; i <
-		 * fl.length; i++) { System.out.println(fl[i]); }
-		 */
+//		
+//		DataFlavor[] fl = t.getTransferDataFlavors();
+//		for (int i = 0; i < fl.length; i++) {
+//			System.out.println(fl[i]);
+//		}
+
 		final IDataFlavorHandler handler = getFlavorHandler(t);
 		paste(t, handler, target, asSibling, isLeft, dropAction);
 	}
@@ -729,7 +730,7 @@ public class MClipboardController extends ClipboardController {
 			final ModeController modeController = Controller.getCurrentModeController();
 			if (!asSibling && modeController.getMapController().isFolded(target)
 			        && ResourceController.getResourceController().getBooleanProperty(RESOURCE_UNFOLD_ON_PASTE)) {
-				modeController.getMapController().setFoldedAndScroll(target, false);
+				modeController.getMapController().unfoldAndScroll(target);
 			}
 			for (final NodeModel child : newNodes) {
 				AttributeController.getController().performRegistrySubtreeAttributes(child);
