@@ -31,7 +31,7 @@ public class MindIconFactory {
 	 * Constructs a MindIcon with the given name from the property file.
 	 * The name of the icon is the file name without the extension.
 	 */
-	public static MindIcon create(final String name) {
+	public static MindIcon createIcon(final String name) {
 		final String translationKeyLabel = name.indexOf('/') > 0 ? "" : String.format(DESC_KEY, name);
 		return new MindIcon(name, name + ".png", translationKeyLabel);
 	}
