@@ -37,12 +37,13 @@ import org.freeplane.core.ui.LengthUnits;
 import org.freeplane.core.util.ColorUtils;
 import org.freeplane.core.util.FreeplaneVersion;
 import org.freeplane.core.util.Quantity;
+import org.freeplane.features.DashVariant;
 import org.freeplane.features.map.MapWriter;
 import org.freeplane.features.map.NodeBuilder;
 import org.freeplane.features.map.NodeModel;
 import org.freeplane.features.map.NodeWriter;
 import org.freeplane.features.nodestyle.NodeStyleModel.Shape;
-import org.freeplane.features.nodestyle.NodeStyleModel.TextAlign;
+import org.freeplane.features.nodestyle.NodeStyleModel.HorizontalTextAlignment;
 import org.freeplane.n3.nanoxml.XMLElement;
 
 class NodeStyleBuilder implements IElementDOMHandler, IExtensionElementWriter, IExtensionAttributeWriter,
@@ -204,6 +205,8 @@ class NodeStyleBuilder implements IElementDOMHandler, IExtensionElementWriter, I
 			reader.addAttributeHandler(NodeBuilder.XML_STYLENODE, "TEMPLATE", formatHandler);
 		}
 
+		// save to 1.1: MAX_WIDTH="200" MAX_WIDTH_QUANTITY="200.0 px"
+		// save: MAX_WIDTH="200.0 px"
 		final IAttributeHandler nodeMaxNodeWidthQuantityHandler = new IAttributeHandler() {
 			public void setAttribute(final Object userObject, final String value) {
 				final NodeModel node = (NodeModel) userObject;
@@ -225,6 +228,16 @@ class NodeStyleBuilder implements IElementDOMHandler, IExtensionElementWriter, I
 		reader.addAttributeHandler(NodeBuilder.XML_NODE, "MAX_WIDTH", nodeMaxNodeWidthHandler);
 		reader.addAttributeHandler(NodeBuilder.XML_STYLENODE, "MAX_WIDTH", nodeMaxNodeWidthHandler);
 
+		final IAttributeHandler nodeIconSizeHandler = new IAttributeHandler() {
+			public void setAttribute(final Object userObject, final String value) {
+				final NodeModel node = (NodeModel) userObject;
+				Quantity<LengthUnits> iconSize = Quantity.fromString(value, LengthUnits.px);
+				node.getSharedData().getIcons().setIconSize(iconSize);
+			}
+		};
+		reader.addAttributeHandler(NodeBuilder.XML_NODE, "ICON_SIZE", nodeIconSizeHandler);
+		reader.addAttributeHandler(NodeBuilder.XML_STYLENODE, "ICON_SIZE", nodeIconSizeHandler);
+
 		final IAttributeHandler nodeMinNodeWidthQuantityHandler = new IAttributeHandler() {
 			public void setAttribute(final Object userObject, final String value) {
 				final NodeModel node = (NodeModel) userObject;
@@ -249,11 +262,75 @@ class NodeStyleBuilder implements IElementDOMHandler, IExtensionElementWriter, I
 		final IAttributeHandler textAlignHandler = new IAttributeHandler() {
 			public void setAttribute(final Object userObject, final String value) {
 				final NodeModel node = (NodeModel) userObject;
-				NodeStyleModel.setTextAlign(node, TextAlign.valueOf(value));
+				NodeStyleModel.setHorizontalTextAlignment(node, HorizontalTextAlignment.valueOf(value));
 			}
 		};
 		reader.addAttributeHandler(NodeBuilder.XML_NODE, "TEXT_ALIGN", textAlignHandler);
 		reader.addAttributeHandler(NodeBuilder.XML_STYLENODE, "TEXT_ALIGN", textAlignHandler);
+
+		final IAttributeHandler borderWidthMatchesEdgeWidthHandler = new IAttributeHandler() {
+			public void setAttribute(final Object userObject, final String value) {
+				final NodeModel node = (NodeModel) userObject;
+				NodeBorderModel.setBorderWidthMatchesEdgeWidth(node, Boolean.valueOf(value));
+			}
+		};
+		reader.addAttributeHandler(NodeBuilder.XML_NODE, "BORDER_WIDTH_LIKE_EDGE", borderWidthMatchesEdgeWidthHandler);
+		reader.addAttributeHandler(NodeBuilder.XML_STYLENODE, "BORDER_WIDTH_LIKE_EDGE", borderWidthMatchesEdgeWidthHandler);
+
+		final IAttributeHandler borderWidthHandler = new IAttributeHandler() {
+			public void setAttribute(final Object userObject, final String value) {
+				final NodeModel node = (NodeModel) userObject;
+				NodeBorderModel.setBorderWidth(node, Quantity.fromString(value, LengthUnits.px));
+			}
+		};
+		reader.addAttributeHandler(NodeBuilder.XML_NODE, "BORDER_WIDTH", borderWidthHandler);
+		reader.addAttributeHandler(NodeBuilder.XML_STYLENODE, "BORDER_WIDTH", borderWidthHandler);
+
+		final IAttributeHandler borderDashMatchesEdgeDashHandler = new IAttributeHandler() {
+			public void setAttribute(final Object userObject, final String value) {
+				final NodeModel node = (NodeModel) userObject;
+				NodeBorderModel.setBorderDashMatchesEdgeDash(node, Boolean.valueOf(value));
+			}
+		};
+		reader.addAttributeHandler(NodeBuilder.XML_NODE, "BORDER_DASH_LIKE_EDGE", borderDashMatchesEdgeDashHandler);
+		reader.addAttributeHandler(NodeBuilder.XML_STYLENODE, "BORDER_DASH_LIKE_EDGE", borderDashMatchesEdgeDashHandler);
+
+		final IAttributeHandler borderDashHandler = new IAttributeHandler() {
+			public void setAttribute(final Object userObject, final String value) {
+				final NodeModel node = (NodeModel) userObject;
+				NodeBorderModel.setBorderDash(node, DashVariant.valueOf(value));
+			}
+		};
+		reader.addAttributeHandler(NodeBuilder.XML_NODE, "BORDER_DASH", borderDashHandler);
+		reader.addAttributeHandler(NodeBuilder.XML_STYLENODE, "BORDER_DASH", borderDashHandler);
+
+		final IAttributeHandler borderColorMatchesEdgeColorHandler = new IAttributeHandler() {
+			public void setAttribute(final Object userObject, final String value) {
+				final NodeModel node = (NodeModel) userObject;
+				NodeBorderModel.setBorderColorMatchesEdgeColor(node, Boolean.valueOf(value));
+			}
+		};
+		reader.addAttributeHandler(NodeBuilder.XML_NODE, "BORDER_COLOR_LIKE_EDGE", borderColorMatchesEdgeColorHandler);
+		reader.addAttributeHandler(NodeBuilder.XML_STYLENODE, "BORDER_COLOR_LIKE_EDGE", borderColorMatchesEdgeColorHandler);
+
+		final IAttributeHandler borderColorHandler = new IAttributeHandler() {
+			public void setAttribute(final Object userObject, final String value) {
+				final NodeModel node = (NodeModel) userObject;
+				NodeBorderModel.setBorderColor(node, ColorUtils.stringToColor(value, NodeBorderModel.getBorderColor(node)));
+			}
+		};
+		final IAttributeHandler borderColorAlphaHandler = new IAttributeHandler() {
+			public void setAttribute(final Object userObject, final String value) {
+				final NodeModel node = (NodeModel) userObject;
+				NodeBorderModel.setBorderColor(node, ColorUtils.alphaToColor(value, NodeBorderModel.getBorderColor(node)));
+			}
+		};
+
+		
+		reader.addAttributeHandler(NodeBuilder.XML_NODE, "BORDER_COLOR", borderColorHandler);
+		reader.addAttributeHandler(NodeBuilder.XML_NODE, "BORDER_COLOR_ALPHA", borderColorAlphaHandler);
+		reader.addAttributeHandler(NodeBuilder.XML_STYLENODE, "BORDER_COLOR", borderColorHandler);
+		reader.addAttributeHandler(NodeBuilder.XML_STYLENODE, "BORDER_COLOR_ALPHA", borderColorAlphaHandler);
 	}
 
 	/**
@@ -268,6 +345,7 @@ class NodeStyleBuilder implements IElementDOMHandler, IExtensionElementWriter, I
 		writer.addExtensionElementWriter(NodeStyleModel.class, this);
 		writer.addExtensionAttributeWriter(NodeStyleModel.class, this);
 		writer.addExtensionAttributeWriter(NodeSizeModel.class, this);
+		writer.addExtensionAttributeWriter(NodeBorderModel.class, this);
 	}
 
 	public void setAttributes(final String tag, final Object node, final XMLElement attributes) {
@@ -281,6 +359,7 @@ class NodeStyleBuilder implements IElementDOMHandler, IExtensionElementWriter, I
 		final NodeModel node = (NodeModel) userObject;
 		writeAttributes(writer, node, (NodeStyleModel)null, true);
 		writeAttributes(writer, node, (NodeSizeModel)null, true);
+		writeAttributes(writer, node, (NodeBorderModel)null, true);
 	}
 
 	public void writeAttributes(final ITreeWriter writer, final Object userObject, final IExtension extension) {
@@ -288,14 +367,20 @@ class NodeStyleBuilder implements IElementDOMHandler, IExtensionElementWriter, I
 		if (forceFormatting) {
 			return;
 		}
+		final NodeModel node = (NodeModel)userObject;
 		if(extension instanceof NodeStyleModel){
 			final NodeStyleModel style = (NodeStyleModel) extension;
-			writeAttributes(writer, null, style, false);
+			writeAttributes(writer, node, style, false);
 			return;
 		}
 		if(extension instanceof NodeSizeModel){
 			final NodeSizeModel size = (NodeSizeModel) extension;
-			writeAttributes(writer, null, size, false);
+			writeAttributes(writer, node, size, false);
+			return;
+		}
+		if(extension instanceof NodeBorderModel){
+			final NodeBorderModel border = (NodeBorderModel) extension;
+			writeAttributes(writer, null, border, false);
 			return;
 		}
 
@@ -328,18 +413,19 @@ class NodeStyleBuilder implements IElementDOMHandler, IExtensionElementWriter, I
 		if (uniformShape) {
 			writer.addAttribute("UNIFORM_SHAPE", "true");
 		}
-		final Boolean numbered = forceFormatting ? nsc.getNodeNumbering(node) : style.getNodeNumbering();
-		if (numbered != null && numbered) {
-			writer.addAttribute("NUMBERED", numbered.toString());
+		final Boolean numbered = forceFormatting ? Boolean.valueOf(nsc.getNodeNumbering(node)) : style.getNodeNumbering();
+		if (numbered != null) {
+			writer.addAttribute("NUMBERED", Boolean.toString(numbered));
 		}
 		final String format = forceFormatting ? nsc.getNodeFormat(node) : style.getNodeFormat();
 		if (format != null) {
 			writer.addAttribute("FORMAT", format);
 		}
-		final TextAlign textAlign = forceFormatting ? nsc.getTextAlign(node) : style.getTextAlign();
-		if (textAlign != null) {
-			writer.addAttribute("TEXT_ALIGN", textAlign.toString());
+		final HorizontalTextAlignment textAlignment = forceFormatting ? nsc.getHorizontalTextAlignment(node) : style.getHorizontalTextAlignment();
+		if (textAlignment != null) {
+			writer.addAttribute("TEXT_ALIGN", textAlignment.toString());
 		}
+
 	}
 
 	private void writeAttributes(final ITreeWriter writer, final NodeModel node, final NodeSizeModel size,
@@ -354,6 +440,35 @@ class NodeStyleBuilder implements IElementDOMHandler, IExtensionElementWriter, I
 			BackwardCompatibleQuantityWriter.forWriter(writer).writeQuantity("MIN_WIDTH", minTextWidth);
 		}
 	}
+	
+	private void writeAttributes(final ITreeWriter writer, final NodeModel node, final NodeBorderModel border,
+			final boolean forceFormatting) {
+		final Boolean borderWidthMatchesEdgeWidth = forceFormatting ? nsc.getBorderWidthMatchesEdgeWidth(node) : border.getBorderWidthMatchesEdgeWidth();
+		if (borderWidthMatchesEdgeWidth != null) {
+			writer.addAttribute("BORDER_WIDTH_LIKE_EDGE", borderWidthMatchesEdgeWidth.toString());
+		}
+		final Quantity<LengthUnits> borderWidth = forceFormatting ? nsc.getBorderWidth(node) : border.getBorderWidth();
+		if (borderWidth != null) {
+			writer.addAttribute("BORDER_WIDTH", borderWidth.toString());
+		}
+		final Boolean borderColorMatchesEdgeColor = forceFormatting ? nsc.getBorderColorMatchesEdgeColor(node) : border.getBorderColorMatchesEdgeColor();
+		if (borderColorMatchesEdgeColor != null) {
+			writer.addAttribute("BORDER_COLOR_LIKE_EDGE", borderColorMatchesEdgeColor.toString());
+		}
+		final Color borderColor = forceFormatting ? nsc.getBorderColor(node) : border.getBorderColor();
+		if (borderColor != null) {
+			ColorUtils.addColorAttributes(writer, "BORDER_COLOR", "BORDER_COLOR_ALPHA", borderColor);
+		}
+		final Boolean borderDashMatchesEdgeDash = forceFormatting ? nsc.getBorderDashMatchesEdgeDash(node) : border.getBorderDashMatchesEdgeDash();
+		if (borderDashMatchesEdgeDash != null) {
+			writer.addAttribute("BORDER_DASH_LIKE_EDGE", borderDashMatchesEdgeDash.toString());
+		}
+		DashVariant borderDash = forceFormatting ? nsc.getBorderDash(node) : border.getBorderDash();
+		if (borderDash != null) {
+			writer.addAttribute("BORDER_DASH", borderDash.name());
+		}
+	}
+	
 	public void writeContent(final ITreeWriter writer, final Object userObject, final String tag) throws IOException {
 		final boolean forceFormatting = Boolean.TRUE.equals(writer.getHint(MapWriter.WriterHint.FORCE_FORMATTING));
 		if (!forceFormatting) {
