@@ -22,7 +22,6 @@ package org.freeplane.features.text;
 import java.awt.Color;
 import java.awt.Component;
 import java.awt.Font;
-import java.util.Collection;
 import java.util.Collections;
 import java.util.LinkedList;
 import java.util.List;
@@ -47,9 +46,6 @@ import org.freeplane.features.mode.Controller;
 import org.freeplane.features.mode.ModeController;
 import org.freeplane.features.nodestyle.NodeSizeModel;
 import org.freeplane.features.nodestyle.NodeStyleController;
-import org.freeplane.features.nodestyle.NodeStyleModel;
-import org.freeplane.features.styles.IStyle;
-import org.freeplane.features.styles.LogicalStyleController;
 import org.freeplane.features.styles.MapStyleModel;
 import org.freeplane.view.swing.map.MainView;
 
@@ -71,6 +67,7 @@ public class TextController implements IExtension {
 	public static final String MARK_TRANSFORMED_TEXT = "highlight_formulas";
 
 
+
 	public static boolean isMarkTransformedTextSet() {
 		return Controller.getCurrentController().getResourceController().getBooleanProperty(MARK_TRANSFORMED_TEXT);
     }
@@ -89,9 +86,8 @@ public class TextController implements IExtension {
 		    new NodeTextConditionController());
 	}
 
-	public static void install( final TextController textController) {
-		final ModeController modeController = Controller.getCurrentModeController();
-		modeController.addExtension(TextController.class, textController);
+	public void install(final ModeController modeController) {
+		modeController.addExtension(TextController.class, this);
 	}
 
 	public TextController(final ModeController modeController) {
@@ -138,7 +134,7 @@ public class TextController implements IExtension {
 		if(object instanceof String){
 			String string = (String) object;
 			if(string.length() > 0 && string.charAt(0) == '\''){
-				if(isTextFormattingDisabled(nodeModel))
+				if(nodeModel != null && extension == nodeModel.getUserObject() && isTextFormattingDisabled(nodeModel))
 					return string;
 				else
 					return string.substring(1);
@@ -161,6 +157,20 @@ public class TextController implements IExtension {
 			return object;
 	}
 	
+	public boolean isFormula(Object object, final NodeModel nodeModel, Object extension) {
+		if(object instanceof String){
+			String string = (String) object;
+			if(string.length() > 0 && string.charAt(0) == '\''){
+				return false;
+			}
+		}
+		for (IContentTransformer textTransformer : getTextTransformers()) {
+			if(textTransformer.isFormula(this, object, nodeModel, extension))
+				return true;
+		}
+		return false;
+	}
+
 	public Icon getIcon(Object object, final NodeModel nodeModel, Object extension){
 		if(object instanceof HighlightedTransformedObject){
 			return getIcon(((HighlightedTransformedObject)object).getObject(), nodeModel, extension);
@@ -178,19 +188,19 @@ public class TextController implements IExtension {
 	}
 	
 	/** returns an error message instead of a normal result if something goes wrong. */
-	public Object getTransformedObjectNoThrow(Object data, final NodeModel node, Object extension) {
+	public Object getTransformedObjectNoFormattingNoThrow(Object data, final NodeModel node, Object extension) {
 		try {
-			return getTransformedObject(data, node, extension);
+			final Object transformedObject = getTransformedObject(data, node, extension);
+			if(transformedObject  instanceof HighlightedTransformedObject)
+				return ((HighlightedTransformedObject)transformedObject).getObject();
+			else
+				return transformedObject;
 		}
 		catch (Throwable e) {
 			LogUtils.warn(e.getMessage(), e);
 			return TextUtils.format("MainView.errorUpdateText", data, e.getLocalizedMessage());
 		}
 	}
-	
-	public Object getTransformedObjectNoFormattingNoThrow(Object data, NodeModel node) {
-		return getTransformedObjectNoThrow(data, node, null);
-	}
 
 	
 	public Object getTransformedObject(NodeModel node)  throws TransformationException{
@@ -198,10 +208,9 @@ public class TextController implements IExtension {
 		return getTransformedObject(userObject, node, userObject);
 	}
 	
-
 	public Object getTransformedObjectNoThrow(NodeModel node) {
 		final Object userObject = node.getUserObject();
-		return getTransformedObjectNoThrow(userObject, node, userObject);
+		return getTransformedObjectNoFormattingNoThrow(userObject, node, userObject);
 	}
 
 	/** convenience method for getTransformedText().toString. */
@@ -211,7 +220,7 @@ public class TextController implements IExtension {
 	}
 	
 	public String getTransformedTextNoThrow(Object text, final NodeModel nodeModel, Object extension) {
-		text = getTransformedObjectNoThrow(text, nodeModel, extension);
+		text = getTransformedObjectNoFormattingNoThrow(text, nodeModel, extension);
 		return text.toString();
 	}
 
@@ -286,7 +295,10 @@ public class TextController implements IExtension {
 
 	private void registerDetailsTooltip() {
 		modeController.addToolTipProvider(DETAILS_TOOLTIP, new ITooltipProvider() {
-				public String getTooltip(ModeController modeController, NodeModel node, Component view) {
+			public String getTooltip(final ModeController modeController, NodeModel node, Component view){
+				return getTooltip(modeController, node, (MainView)view);
+			}
+			private String getTooltip(final ModeController modeController, NodeModel node, MainView view) {
 					final DetailTextModel detailText = DetailTextModel.getDetailText(node);
 					if (detailText == null || ! (detailText.isHidden() || ShortenedTextModel.isShortened(node)) ){
 						 return null;
@@ -297,7 +309,9 @@ public class TextController implements IExtension {
 			        Font detailFont = style.getFont(detailStyleNode);
 			        Color detailBackground = style.getBackgroundColor(detailStyleNode);
 			        Color detailForeground = style.getColor(detailStyleNode);
-			        final int alignment = style.getTextAlign(detailStyleNode).swingConstant;
+			        final int alignment = style.getHorizontalTextAlignment(detailStyleNode).swingConstant;
+			        
+			        float zoom = view.getNodeView().getMap().getZoom();
 					
 					final StringBuilder htmlBodyStyle = new StringBuilder("<body><div style=\"")
 							.append(new CssRuleBuilder()
@@ -305,7 +319,7 @@ public class TextController implements IExtension {
 							.withColor(detailForeground)
 							.withBackground(detailBackground)
 							.withAlignment(alignment)
-							.withMaxWidthAsPt(NodeSizeModel.getMaxNodeWidth(detailStyleNode), style.getMaxWidth(node)))
+							.withMaxWidthAsPt(zoom, NodeSizeModel.getMaxNodeWidth(detailStyleNode), style.getMaxWidth(node)))
 							.append("\">");
 					
 					String noteText= detailText.getHtml();
@@ -327,12 +341,13 @@ public class TextController implements IExtension {
 				    }
 				    final NodeStyleController style = (NodeStyleController) modeController.getExtension(NodeStyleController.class);
 				    final Font font = style.getFont(node);
+				    float zoom = view.getNodeView().getMap().getZoom();
 					final StringBuilder htmlBodyStyle = new StringBuilder("<body><div style=\"")
 							.append(new CssRuleBuilder().withHTMLFont(font)
 							.withColor(view.getForeground())
 							.withBackground(view.getNodeView().getTextBackground())
 							.withAlignment(view.getHorizontalAlignment())
-							.withMaxWidthAsPt(style.getMaxWidth(node)));
+							.withMaxWidthAsPt(zoom, style.getMaxWidth(node)));
 				    final Object data = node.getUserObject();
 				    String text;
 				    try {
@@ -370,43 +385,18 @@ public class TextController implements IExtension {
 		Controller.getCurrentModeController().getMapController().nodeChanged(node, "SHORTENER", oldState, shortened);   
 	}
 
-	public String getNodeFormat(NodeModel node) {
-		Collection<IStyle> collection = LogicalStyleController.getController(modeController).getStyles(node);
-		final MapStyleModel model = MapStyleModel.getExtension(node.getMap());
-		for(IStyle styleKey : collection){
-			final NodeModel styleNode = model.getStyleNode(styleKey);
-			if (styleNode == null) {
-				continue;
-			}
-			final String format = NodeStyleModel.getNodeFormat(styleNode);
-			if (format != null) {
-				return format;
-			}
-        } 
-		// do not return PatternFormat.IDENTITY_PATTERN if parse_data=false because that would
-		// automatically disable all IContentTransformers!
-		return PatternFormat.STANDARD_FORMAT_PATTERN;
-    }
-
 	public boolean parseData() {
         return false;
     }
 
-    public boolean getNodeNumbering(NodeModel node) {
-		Collection<IStyle> collection = LogicalStyleController.getController(modeController).getStyles(node);
-		final MapStyleModel model = MapStyleModel.getExtension(node.getMap());
-		for(IStyle styleKey : collection){
-			final NodeModel styleNode = model.getStyleNode(styleKey);
-			if (styleNode == null) {
-				continue;
-			}
-			final Boolean numbering = NodeStyleModel.getNodeNumbering(styleNode);
-			if (numbering != null) {
-				return numbering;
-			}
-		}
-		return false;
+	public String getNodeFormat(NodeModel node) {
+		return modeController.getExtension(NodeStyleController.class).getNodeFormat(node);
     }
+
+    public boolean getNodeNumbering(NodeModel node) {
+    	return modeController.getExtension(NodeStyleController.class).getNodeNumbering(node);    
+    }
+    
 	public ModeController getModeController() {
     	return modeController;
     }
@@ -414,4 +404,5 @@ public class TextController implements IExtension {
 	public boolean canEdit() {
 		return false;
 	}
+
 }
