@@ -24,15 +24,20 @@ import java.awt.BorderLayout;
 import java.awt.Component;
 import java.awt.Container;
 import java.awt.Cursor;
+import java.awt.Desktop;
 import java.awt.EventQueue;
+import java.awt.Frame;
 import java.awt.HeadlessException;
+import java.awt.Window;
 import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
+import java.net.URL;
 import java.util.Collections;
 import java.util.Set;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
 
+import javax.swing.ImageIcon;
 import javax.swing.JApplet;
 import javax.swing.JComponent;
 import javax.swing.JOptionPane;
@@ -47,19 +52,18 @@ import org.freeplane.features.filter.NextNodeAction;
 import org.freeplane.features.filter.NextPresentationItemAction;
 import org.freeplane.features.format.FormatController;
 import org.freeplane.features.help.HelpController;
+import org.freeplane.features.highlight.HighlightController;
 import org.freeplane.features.icon.IconController;
 import org.freeplane.features.link.LinkController;
 import org.freeplane.features.map.MapController;
 import org.freeplane.features.map.MapController.Direction;
 import org.freeplane.features.mode.Controller;
-import org.freeplane.features.mode.browsemode.BModeController;
 import org.freeplane.features.print.PrintController;
 import org.freeplane.features.styles.LogicalStyleFilterController;
 import org.freeplane.features.styles.MapViewLayout;
 import org.freeplane.features.text.TextController;
 import org.freeplane.features.time.TimeController;
 import org.freeplane.features.ui.FrameController;
-import org.freeplane.main.browsemode.BModeControllerFactory;
 import org.freeplane.view.swing.features.nodehistory.NodeHistory;
 import org.freeplane.view.swing.map.MapViewController;
 import org.freeplane.view.swing.map.ViewLayoutTypeAction;
@@ -67,8 +71,12 @@ import org.freeplane.view.swing.map.ViewLayoutTypeAction;
 public class FreeplaneApplet extends JApplet {
 	
 	@SuppressWarnings("serial")
-	private class GlassPane extends JComponent{
-		public GlassPane() {
+	private static class GlassPane extends JComponent{
+		private final Controller controller;
+
+
+		public GlassPane(Controller controller) {
+			this.controller = controller;
 			addMouseListener(new MouseAdapter(){});
 		}
 
@@ -94,15 +102,16 @@ public class FreeplaneApplet extends JApplet {
 		}
 	}
 	
-	private AppletResourceController appletResourceController;
 	/**
 	 * 
 	 */
 	private static final long serialVersionUID = 1L;
 	private AppletViewController appletViewController;
  	private Controller controller;
+	private static boolean instanceInitialized = false;
  	
  	final static Lock appletLock = new ReentrantLock();
+	private Boolean isLaunchedByJavaWebStart;
 
 	public FreeplaneApplet() throws HeadlessException {
 	    super();
@@ -115,73 +124,109 @@ public class FreeplaneApplet extends JApplet {
 	@SuppressWarnings("serial")
     @Override
 	public void init() {
-		try{
-			appletLock.lock();
-			appletResourceController = new AppletResourceController(this);
-			if (appletResourceController == null) {
-				appletResourceController = new AppletResourceController(this);
+		configureFrame();
+		new Thread(new Runnable() {
+			
+			@Override
+			public void run() {
+				try{
+					appletLock.lock();
+					AppletResourceController appletResourceController = new AppletResourceController(FreeplaneApplet.this);
+					new ParserDelegator(){
+						{
+							setDefaultDTD();
+						}
+					};
+					controller = new Controller(appletResourceController);
+					updateLookAndFeel(appletResourceController);
+					Controller.setCurrentController(controller);
+					final Container contentPane = getContentPane();
+					contentPane.setLayout(new BorderLayout());
+					MapViewController mapViewController = new MapViewController(controller);
+					appletViewController = new AppletViewController(FreeplaneApplet.this, controller, mapViewController);
+					controller.addAction(new ViewLayoutTypeAction(MapViewLayout.OUTLINE));
+					controller.addExtension(HighlightController.class, new HighlightController());
+					FilterController.install();
+					PrintController.install();
+					HelpController.install();
+					NodeHistory.install(controller);
+					FormatController.install(new FormatController());
+					ModelessAttributeController.install();
+					TextController.install();
+					MapController.install();
+
+					TimeController.install();
+					LinkController.install();
+					IconController.install();
+					FilterController.getCurrentFilterController().getConditionFactory().addConditionController(70,
+					    new LogicalStyleFilterController());
+					final BModeController browseController = BModeControllerFactory.createModeController();
+					final Set<String> emptySet = Collections.emptySet();
+					FilterController.getController(controller).loadDefaultConditions();
+					controller.addAction(new ShowSelectionAsRectangleAction());
+					controller.addAction(new NextNodeAction(Direction.FORWARD));
+					controller.addAction(new NextNodeAction(Direction.BACK));
+					controller.addAction(new NextNodeAction(Direction.FORWARD_N_FOLD));
+					controller.addAction(new NextNodeAction(Direction.BACK_N_FOLD));
+					controller.addAction(NextPresentationItemAction.createFoldingAction());
+					controller.addAction(NextPresentationItemAction.createNotFoldingAction());
+					browseController.updateMenus("/xml/appletmenu.xml", emptySet);
+					appletResourceController.getAcceleratorManager().loadAcceleratorPresets();
+
+					controller.selectMode(browseController);
+					setPropertyByParameter(appletResourceController, "browsemode_initial_map");
+					isLaunchedByJavaWebStart = isParameterTrue("launched_by_java_web_start");
+					if(isLaunchedByJavaWebStart) {
+						if(instanceInitialized)
+							throw new RuntimeException("singleAppletInstance allowed");
+						else
+							instanceInitialized = true;
+					} else
+						addGlassPane();
+					controller.getViewController().setMenubarVisible(false);
+					SwingUtilities.invokeLater(new Runnable() {
+						
+						@Override
+						public void run() {
+							appletViewController.init(controller);
+						}
+					});
+				}
+				catch(RuntimeException e){
+					e.printStackTrace();
+					throw e;
+				}
+				finally{
+					appletLock.unlock();
+				}
 			}
-			new ParserDelegator(){
-				{
-					setDefaultDTD();
-				}
-			};
-			controller = new Controller(appletResourceController);
-			updateLookAndFeel();
-			Controller.setCurrentController(controller);
-			final Container contentPane = getContentPane();
-			contentPane.setLayout(new BorderLayout());
-			MapViewController mapViewController = new MapViewController(controller);
-			appletViewController = new AppletViewController(this, controller, mapViewController);
-			controller.addAction(new ViewLayoutTypeAction(MapViewLayout.OUTLINE));
-			FilterController.install();
-			PrintController.install();
-			HelpController.install();
-			NodeHistory.install(controller);
-			FormatController.install(new FormatController());
-			ModelessAttributeController.install();
-			TextController.install();
-			MapController.install();
+		}, "Freeplane applet initialization").start();
+	}
 
-			TimeController.install();
-			LinkController.install();
-			IconController.install();
-			FilterController.getCurrentFilterController().getConditionFactory().addConditionController(70,
-			    new LogicalStyleFilterController());
-			final BModeController browseController = BModeControllerFactory.createModeController();
-			final Set<String> emptySet = Collections.emptySet();
-			FilterController.getController(controller).loadDefaultConditions();
-			controller.addAction(new ShowSelectionAsRectangleAction());
-			controller.addAction(new NextNodeAction(Direction.FORWARD));
-			controller.addAction(new NextNodeAction(Direction.BACK));
-			controller.addAction(new NextNodeAction(Direction.FORWARD_N_FOLD));
-			controller.addAction(new NextNodeAction(Direction.BACK_N_FOLD));
-			controller.addAction(new NextPresentationItemAction());
-			browseController.updateMenus("/xml/appletmenu.xml", emptySet);
-			ResourceController.getResourceController().getAcceleratorManager().loadAcceleratorPresets();
+	private Boolean isParameterTrue(String name) {
+		return Boolean.valueOf(getParameter(name));
+	}
 
-			controller.selectMode(browseController);
-			appletResourceController.setPropertyByParameter(this, "browsemode_initial_map");
-			final GlassPane glassPane = new GlassPane();
-			setGlassPane(glassPane);
-			glassPane.setVisible(true);
-			controller.getViewController().setMenubarVisible(false);
-			SwingUtilities.invokeLater(new Runnable() {
-				@Override
-				public void run() {
-					appletViewController.init(controller);
-				}
-			});
-		}
-		catch(RuntimeException e){
-			e.printStackTrace();
-			throw e;
-		}
-		finally{
-			appletLock.unlock();
+	private void configureFrame() {
+		Window window = SwingUtilities.windowForComponent(this);
+		if (window instanceof Frame){
+			Frame frame = (Frame)window;
+			ImageIcon mWindowIcon;
+			mWindowIcon = new ImageIcon(getClass().getResource(
+					"/images/Freeplane_frame_icon_64x64.png"));
+			frame.setIconImage(mWindowIcon.getImage());			
+			if (!frame.isResizable()){
+				frame.setResizable(true);
+			}
 		}
 	}
 
+	private void addGlassPane() {
+		final GlassPane glassPane = new GlassPane(controller);
+		setGlassPane(glassPane);
+		glassPane.setVisible(true);
+	}
+
 	@Override
 	public void start() {
 		EventQueue.invokeLater(new Runnable() {
@@ -196,9 +241,9 @@ public class FreeplaneApplet extends JApplet {
 		super.stop();
 	}
 
-	private void updateLookAndFeel() {
+	private void updateLookAndFeel(ResourceController appletResourceController) {
 		String lookAndFeel = "";
-		appletResourceController.setPropertyByParameter(this, "lookandfeel");
+		setPropertyByParameter(appletResourceController, "lookandfeel");
 		lookAndFeel = appletResourceController.getProperty("lookandfeel");
 		FrameController.setLookAndFeel(lookAndFeel, true);
 	}
@@ -233,4 +278,22 @@ public class FreeplaneApplet extends JApplet {
 		}
 	}
 
+	void setPropertyByParameter(ResourceController resourceController, final String key) {
+		final String val = getParameter(key);
+		if (val != null && val != "") {
+			resourceController.setProperty(key, val);
+		}
+	}
+
+	public void showDocument(URL doc) {
+		if(isLaunchedByJavaWebStart && Desktop.isDesktopSupported())
+			try {
+				Desktop.getDesktop().browse(doc.toURI());
+				return;
+			} catch (Exception e) {
+				e.printStackTrace();
+			}
+		getAppletContext().showDocument(doc, "_blank");
+	}
+
 }
