@@ -43,6 +43,7 @@ import org.freeplane.features.edge.mindmapmode.MEdgeController;
 import org.freeplane.features.filter.FilterController;
 import org.freeplane.features.format.FormatController;
 import org.freeplane.features.format.ScannerController;
+import org.freeplane.features.highlight.HighlightController;
 import org.freeplane.features.icon.IconController;
 import org.freeplane.features.icon.mindmapmode.MIconController;
 import org.freeplane.features.link.LinkController;
@@ -72,7 +73,7 @@ import org.freeplane.features.styles.MapViewLayout;
 import org.freeplane.features.styles.mindmapmode.MLogicalStyleController;
 import org.freeplane.features.styles.mindmapmode.MUIFactory;
 import org.freeplane.features.styles.mindmapmode.ShowFormatPanelAction;
-import org.freeplane.features.styles.mindmapmode.StyleEditorPanel;
+import org.freeplane.features.styles.mindmapmode.styleeditorpanel.StyleEditorPanel;
 import org.freeplane.features.text.TextController;
 import org.freeplane.features.text.mindmapmode.MTextController;
 import org.freeplane.features.ui.ToggleToolbarAction;
@@ -111,6 +112,7 @@ public class SModeControllerFactory {
 		final MapViewController mapViewController = new MMapViewController(controller);
 		final DialogController viewController = new DialogController(controller, mapViewController, dialog);
 		controller.setViewController(viewController);
+		controller.addExtension(HighlightController.class, new HighlightController());
 		controller.addAction(currentController.getAction("AboutAction"));
 		FilterController.install();
 		TextController.install();
@@ -129,9 +131,9 @@ public class SModeControllerFactory {
 		modeController.setUserInputListenerFactory(userInputListenerFactory);
 		controller.addExtension(ModelessAttributeController.class, new ModelessAttributeController());
 		new MMapController(modeController);
-		TextController.install(new MTextController(modeController));
+		new MTextController(modeController).install(modeController);
 		SpellCheckerController.install(modeController);
-		IconController.install(new MIconController(modeController));
+		new MIconController(modeController).install(modeController);
 		NodeStyleController.install(new MNodeStyleController(modeController));
 		LocationController.install(new MLocationController());
 		EdgeController.install(new MEdgeController(modeController));
