@@ -21,12 +21,15 @@ package org.freeplane.view.swing.map;
 
 import java.awt.Color;
 import java.awt.Component;
+import java.awt.Dimension;
 import java.awt.EventQueue;
 import java.awt.Font;
 import java.awt.Graphics2D;
 import java.awt.KeyboardFocusManager;
+import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.RenderingHints;
+import java.awt.SystemColor;
 import java.awt.image.BufferedImage;
 import java.awt.image.RenderedImage;
 import java.io.File;
@@ -41,7 +44,9 @@ import java.util.Map;
 import java.util.Vector;
 
 import javax.swing.DefaultComboBoxModel;
+import javax.swing.FocusManager;
 import javax.swing.JComboBox;
+import javax.swing.JComponent;
 import javax.swing.JOptionPane;
 import javax.swing.JSpinner;
 import javax.swing.SpinnerNumberModel;
@@ -53,9 +58,11 @@ import org.freeplane.core.resources.IFreeplanePropertyListener;
 import org.freeplane.core.resources.ResourceController;
 import org.freeplane.core.ui.components.JComboBoxWithBorder;
 import org.freeplane.core.ui.components.UITools;
+import org.freeplane.core.ui.image.BigBufferedImage;
 import org.freeplane.core.util.TextUtils;
 import org.freeplane.features.map.IMapLifeCycleListener;
 import org.freeplane.features.map.IMapSelection;
+import org.freeplane.features.map.IMapSelection.NodePosition;
 import org.freeplane.features.map.IMapSelectionListener;
 import org.freeplane.features.map.MapController;
 import org.freeplane.features.map.MapModel;
@@ -285,7 +292,7 @@ public class MapViewController implements IMapViewManager , IMapViewChangeListen
 				return true;
 			}
 			else
-				return mapController.close(map);
+				return map.close();
 			
 		}
 		map.removeMapChangeListener(mapView);
@@ -325,18 +332,63 @@ public class MapViewController implements IMapViewManager , IMapViewChangeListen
 		}
 		view.preparePrinting();
 		final Rectangle innerBounds = view.getInnerBounds();
+		return createImage(dpi, innerBounds);
+	}
 
+	public RenderedImage createImage(final Dimension slideSize, NodeModel placedNode, NodePosition placedNodePosition, int dpi) {
+		final MapView view = getMapView();
+		if (view == null) {
+			return null;
+		}
+		final NodeView placedNodeView = view.getNodeView(placedNode);
+		if (placedNodeView == null) {
+			return createImage(dpi);
+		}
+
+		view.preparePrinting();
+		final JComponent content = placedNodeView.getContent();
+		Point contentLocation = new Point();
+		UITools.convertPointToAncestor(content, contentLocation, view);
+		final Rectangle printedGraphicsBounds = new Rectangle(contentLocation.x + content.getWidth() / 2 - slideSize.width / 2, 
+				contentLocation.y + content.getHeight() / 2 - slideSize.height
+				/ 2, slideSize.width, slideSize.height);
+		
+		final int distanceToMargin = (slideSize.width - content.getWidth()) / 2 - 10;
+		if(placedNodePosition == NodePosition.WEST){
+			printedGraphicsBounds.x += distanceToMargin;
+		}
+		if(placedNodePosition == NodePosition.EAST){
+			printedGraphicsBounds.x -= distanceToMargin;
+		}
+		return createImage(dpi, printedGraphicsBounds);
+	}
+
+	public RenderedImage createImage(int dpi, final Rectangle printedArea) {
+		final MapView view = getMapView();
+		view.preparePrinting();
+		final BufferedImage myImage = printToImage(dpi, view, printedArea);
+		view.endPrinting();
+		return myImage;
+	}
+
+	private BufferedImage printToImage(int dpi, final MapView view, final Rectangle innerBounds) {
 		double scaleFactor = (double) dpi / (double) (UITools.FONT_SCALE_FACTOR * 72);
 
 		int imageWidth = (int) Math.ceil(innerBounds.width * scaleFactor);
 		int imageHeight = (int) Math.ceil(innerBounds.height * scaleFactor);
 
-		final BufferedImage myImage = (BufferedImage) view.createImage(imageWidth, imageHeight);
+		final BufferedImage myImage = BigBufferedImage.create(imageWidth, imageHeight, BufferedImage.TYPE_INT_RGB);
 		final Graphics2D g = (Graphics2D) myImage.getGraphics();
+		Color background = view.getBackground();
+        if(background == null) {
+            background = SystemColor.window;
+        }
+
+		g.setBackground(background);
+		g.clearRect(0, 0, imageWidth, imageHeight);
 		g.scale(scaleFactor, scaleFactor);
 		g.translate(-innerBounds.x, -innerBounds.y);
 		view.print(g);
-		view.endPrinting();
 		return myImage;
 	}
 
@@ -375,6 +427,23 @@ public class MapViewController implements IMapViewManager , IMapViewChangeListen
 			return node.isFolded();
 		return nodeView.isFolded();
 	}
+	
+
+	@Override
+	public void displayOnCurrentView(NodeModel node) {
+		if(selectedMapView != null)
+			selectedMapView.display(node);
+	}
+	
+	public void setFoldedOnCurrentView(NodeModel node, boolean folded){
+		if(selectedMapView == null || ! node.hasChildren())
+			return;
+		final NodeView nodeView = selectedMapView.getNodeView(node);
+		if(nodeView == null)
+			return;
+		nodeView.setFolded(folded);
+		
+	}
 
 	/* (non-Javadoc)
 	 * @see org.freeplane.core.frame.IMapViewController#getFont(org.freeplane.core.model.NodeModel)
@@ -789,7 +858,8 @@ public class MapViewController implements IMapViewManager , IMapViewChangeListen
 		return zoomValue;
 	}
 
-	private static final String[] zooms = { "25%", "50%", "75%", "100%", "150%", "200%", "300%", "400%" };
+	private static final String[] zooms = { "25%", "50%", "75%", "100%", 
+			"150%", "200%", "300%", "400%", "600%", "800%", "1200%", "1600%" , "2400%", "3200%" };
 	public void obtainFocusForSelected() {
 		SwingUtilities.invokeLater(new Runnable() {
 			public void run() {
@@ -913,4 +983,76 @@ public class MapViewController implements IMapViewManager , IMapViewChangeListen
 		ResourceController.getResourceController().setProperty("antialiasAll", (antialiasAll ? "true" : "false"));
 	}
 
+	@Override
+	public void moveFocusFromDescendantToSelection(Component ancestor) {
+		Component focusOwner = FocusManager.getCurrentManager().getFocusOwner();
+		boolean toolbarLostFocus = focusOwner != null && SwingUtilities.isDescendingFrom(focusOwner, ancestor);
+		if (toolbarLostFocus) {
+			final Component selectedComponent = getSelectedComponent();
+			if (selectedComponent != null)
+				selectedComponent.requestFocus();
+		}
+	}
+
+	@Override
+	public boolean isChildHidden(NodeModel node) {
+		if(selectedMapView == null)
+			return false;
+		final NodeModel parentNode = node.getParentNode();
+		if(parentNode == null)
+			return false;
+		final NodeView nodeView = selectedMapView.getNodeView(parentNode);
+		if(nodeView == null)
+			return false;
+		return nodeView.isChildHidden(node);
+	}
+
+	@Override
+	public boolean hasHiddenChildren(NodeModel node) {
+		if(selectedMapView == null)
+			return false;
+		final NodeView nodeView = selectedMapView.getNodeView(node);
+		if(nodeView == null)
+			return false;
+		return nodeView.hasHiddenChildren();
+	}
+
+	@Override
+	public boolean unfoldHiddenChildren(NodeModel node) {
+		if(selectedMapView == null)
+			return false;
+		final NodeView nodeView = selectedMapView.getNodeView(node);
+		if(nodeView == null)
+			return false;
+		return nodeView.unfoldHiddenChildren();
+	}
+
+	@Override
+	public void hideChildren(NodeModel node) {
+		if(selectedMapView == null)
+			return; 
+		final NodeView nodeView = selectedMapView.getNodeView(node);
+		if(nodeView == null)
+			return; 
+		nodeView.hideChildren(node);
+	}
+
+	@Override
+	public boolean showHiddenNode(NodeModel node) {
+		if(selectedMapView == null)
+			return false;
+		final NodeModel parentNode = node.getParentNode();
+		if(parentNode == null)
+			return false;
+		final NodeView nodeView = selectedMapView.getNodeView(parentNode);
+		if(nodeView == null)
+			return false;
+		return nodeView.showHiddenNode(node);
+	}
+
+	@Override
+	public boolean isSpotlightEnabled() {
+		return selectedMapView != null && selectedMapView.isSpotlightEnabled();
+	}
+
 }
