@@ -36,8 +36,11 @@ import java.awt.dnd.DropTarget;
 import java.awt.dnd.DropTargetListener;
 import java.awt.event.WindowEvent;
 import java.awt.event.WindowFocusListener;
+import java.util.HashSet;
 import java.util.LinkedList;
+import java.util.List;
 import java.util.ListIterator;
+import java.util.Set;
 
 import javax.swing.JComponent;
 import javax.swing.SwingUtilities;
@@ -47,6 +50,7 @@ import org.freeplane.core.ui.IUserInputListenerFactory;
 import org.freeplane.core.ui.components.UITools;
 import org.freeplane.core.util.LogUtils;
 import org.freeplane.core.util.ObjectRule;
+import org.freeplane.features.DashVariant;
 import org.freeplane.features.attribute.AttributeController;
 import org.freeplane.features.attribute.NodeAttributeTableModel;
 import org.freeplane.features.cloud.CloudController;
@@ -54,14 +58,15 @@ import org.freeplane.features.cloud.CloudModel;
 import org.freeplane.features.edge.EdgeController;
 import org.freeplane.features.edge.EdgeController.Rules;
 import org.freeplane.features.edge.EdgeStyle;
-import org.freeplane.features.filter.FilterController;
+import org.freeplane.features.highlight.HighlightController;
+import org.freeplane.features.highlight.NodeHighlighter;
 import org.freeplane.features.icon.HierarchicalIcons;
 import org.freeplane.features.map.EncryptionModel;
 import org.freeplane.features.map.FreeNode;
-import org.freeplane.features.map.HideChildSubtree;
 import org.freeplane.features.map.HistoryInformationModel;
 import org.freeplane.features.map.INodeView;
 import org.freeplane.features.map.MapChangeEvent;
+import org.freeplane.features.map.MapModel;
 import org.freeplane.features.map.NodeChangeEvent;
 import org.freeplane.features.map.NodeDeletionEvent;
 import org.freeplane.features.map.NodeModel;
@@ -76,7 +81,6 @@ import org.freeplane.features.nodestyle.NodeStyleController;
 import org.freeplane.features.nodestyle.NodeStyleModel;
 import org.freeplane.features.nodestyle.NodeStyleModel.Shape;
 import org.freeplane.features.nodestyle.ShapeConfigurationModel;
-import org.freeplane.features.styles.AutomaticLayoutController;
 import org.freeplane.features.styles.MapViewLayout;
 import org.freeplane.features.text.TextController;
 import org.freeplane.view.swing.map.attribute.AttributeView;
@@ -91,6 +95,7 @@ import org.freeplane.view.swing.map.edge.EdgeViewFactory;
  * TreeCellRenderer).
  */
 public class NodeView extends JComponent implements INodeView {
+	private static final int HIGHLIGHTED_NODE_ARC_MARGIN = 4;
 	final static int ALIGN_BOTTOM = -1;
 	final static int ALIGN_CENTER = 0;
 	final static int ALIGN_TOP = 1;
@@ -129,6 +134,7 @@ public class NodeView extends JComponent implements INodeView {
 	private int topOverlap;
 	private int bottomOverlap;
 	private boolean isFolded;
+	private DashVariant edgeDash = DashVariant.DEFAULT;
 	
 	public static final int DETAIL_VIEWER_POSITION = 2;
 
@@ -788,9 +794,11 @@ public class NodeView extends JComponent implements INodeView {
 	}
 
 	void addChildViews() {
+		if(isFolded)
+			return;
 		int index = 0;
-		for (NodeModel child : getMap().getModeController().getMapController().childrenFolded(getModel())) {
-			if(child.containsExtension(HideChildSubtree.class))
+		for (NodeModel child : getMap().getModeController().getMapController().childrenUnfolded(getModel())) {
+			if(isChildHidden(child))
 				return;
 			if(getComponentCount() <= index
 					|| ! (getComponent(index) instanceof NodeView))
@@ -865,17 +873,11 @@ public class NodeView extends JComponent implements INodeView {
 			return;
 		}
 		final Object property = event.getProperty();
-		if (property == NodeChangeType.FOLDING || property == HideChildSubtree.instance || property == EncryptionModel.class) {
+		if (property == NodeChangeType.FOLDING || property == Properties.HIDDEN_CHILDREN || property == EncryptionModel.class) {
 			if(map.isSelected() || property == EncryptionModel.class && ! isFolded){
-				boolean wasFolded = isFolded;
-				isFolded = getMap().getModeController().getMapController().isFolded(model);
-				if(wasFolded != isFolded || property == HideChildSubtree.instance) {
-					treeStructureChanged();
-					getMap().selectIfSelectionIsEmpty(this);
-					Shape shape = NodeStyleController.getController(getMap().getModeController()).getShape(model);
-					if (shape.equals(NodeStyleModel.Shape.combined))
-						update();
-				}
+				boolean folded = getMap().getModeController().getMapController().isFolded(model);
+				boolean force = property ==Properties.HIDDEN_CHILDREN;
+				setFolded(folded, force);
 			}
 			if(property != EncryptionModel.class)
 				return;
@@ -889,6 +891,12 @@ public class NodeView extends JComponent implements INodeView {
 			revalidate();
 			return;
 		}
+		if (property.equals(NodeModel.NODE_ICON_SIZE))
+		{
+			mainView.updateIcons(this);
+			revalidate();
+			return;
+		}
 
 		if (property.equals(HistoryInformationModel.class)) {
 			return;
@@ -898,6 +906,23 @@ public class NodeView extends JComponent implements INodeView {
 			getParentView().numberingChanged(node.getParentNode().getIndex(node) + 1);
 	}
 
+	public void setFolded(boolean folded) {
+		setFolded(folded, false);
+		revalidate();
+	}
+	
+	private void setFolded(boolean folded, boolean force) {
+		boolean wasFolded = isFolded;
+		this.isFolded = folded;
+		if(wasFolded != isFolded || force) {
+			treeStructureChanged();
+			getMap().selectIfSelectionIsEmpty(this);
+			Shape shape = NodeStyleController.getController(getMap().getModeController()).getShape(model);
+			if (shape.equals(NodeStyleModel.Shape.combined))
+				update();
+		}
+	}
+
 	public void onNodeDeleted(NodeDeletionEvent nodeDeletionEvent) {
 		if (nodeDeletionEvent.index >= getComponentCount() - 1) {
 			return;
@@ -1141,23 +1166,30 @@ public class NodeView extends JComponent implements INodeView {
 		g.translate(origin.x, origin.y);
 		mainView.paintDecoration(this, g);
 		g.translate(-origin.x, -origin.y);
-		final FilterController filterController = FilterController.getController(getMap().getModeController().getController());
-		if(filterController.isNodeHighlighted(getModel())){
-			final Color oldColor = g.getColor();
-			final Stroke oldStroke = g.getStroke();
-			g.setColor(Color.MAGENTA);
-			g.setStroke(getMap().getStandardSelectionStroke());
-			final JComponent content = getContent();
-			Point contentLocation = content.getLocation();
-			final int arcWidth = 8;
-			g.drawRoundRect(contentLocation.x - arcWidth, contentLocation.y - arcWidth, content.getWidth() + 2 * arcWidth,
-			    content.getHeight() + 2 * arcWidth, 15, 15);
-			g.setColor(oldColor);
-			g.setStroke(oldStroke);
+		final HighlightController highlightController = getMap().getModeController().getController().getExtension(HighlightController.class);
+		final List<NodeHighlighter> highlighters = highlightController.getHighlighters(model, map.isPrinting());
+		int margin = HIGHLIGHTED_NODE_ARC_MARGIN;
+		for(NodeHighlighter highlighter : highlighters){
+			margin += HIGHLIGHTED_NODE_ARC_MARGIN;
+			highlightNode(g, highlighter, margin);
 		}
 		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, renderingHint);
 	}
 
+	public void highlightNode(final Graphics2D g, NodeHighlighter highlighter, final int arcMargin) {
+		final Color oldColor = g.getColor();
+		final Stroke oldStroke = g.getStroke();
+		g.setStroke(NodeHighlighter.DEFAULT_STROKE);
+		highlighter.configure(g, map.isPrinting());
+		final JComponent content = getContent();
+		Point contentLocation = content.getLocation();
+		final int arcWidth = 15;
+		g.drawRoundRect(contentLocation.x - arcMargin, contentLocation.y - arcMargin, content.getWidth() + 2 * arcMargin,
+		    content.getHeight() + 2 * arcMargin, arcWidth, arcWidth);
+		g.setColor(oldColor);
+		g.setStroke(oldStroke);
+	}
+
 	/**
 	 * This is a bit problematic, because getChildrenViews() only works if model
 	 * is not yet removed. (So do not _really_ delete the model before the view
@@ -1354,7 +1386,8 @@ public class NodeView extends JComponent implements INodeView {
 		mainView.setVisible(true);
 		mainView.updateTextColor(this);
 		mainView.updateFont(this);
-		mainView.updateTextAlign(this);
+		mainView.updateHorizontalTextAlignment(this);
+		mainView.updateBorder(this);
         MapView map = getMap();
 		final ModeController modeController = map.getModeController();
         final NodeStyleController nsc = NodeStyleController.getController(modeController);
@@ -1403,8 +1436,8 @@ public class NodeView extends JComponent implements INodeView {
         final EdgeController edgeController = EdgeController.getController(getMap().getModeController());
 		this.edgeStyle = edgeController.getStyle(model, false);
 		final NodeModel realNode = SummaryNode.getRealNode(model);
-		final Integer newWidth = edgeController.getWidth(realNode, false);
-		this.edgeWidth = newWidth;
+		this.edgeWidth = edgeController.getWidth(realNode, false);
+		this.edgeDash = edgeController.getDash(realNode, false);
 		final ObjectRule<Color, Rules> newColor = edgeController.getColorRule(realNode);
 		this.edgeColor = newColor;
 		final NodeModel parentNode = model.getParentNode();
@@ -1421,6 +1454,15 @@ public class NodeView extends JComponent implements INodeView {
 		return EdgeStyle.values()[0];
     }
 
+	public DashVariant getEdgeDash() {
+		if(edgeDash != null)
+		    return edgeDash;
+		final NodeView parentView = getParentView();
+		if(parentView != null)
+			return parentView.getEdgeDash();
+		return DashVariant.DEFAULT;
+    }
+	
 	public int getEdgeWidth() {
 		if(edgeWidth != null)
 		    return edgeWidth;
@@ -1447,11 +1489,11 @@ public class NodeView extends JComponent implements INodeView {
 				index = parentNode.getIndex(model) + 1;
 			else
 				index = model.getNodeLevel(false) + (model.isHiddenSummary() ? 1 : 0);
-			ModeController modeController = getMap().getModeController();
-			AutomaticLayoutController automaticLayoutController = modeController.getExtension(AutomaticLayoutController.class);
-			NodeModel styleNode = automaticLayoutController.getStyleNode(map.getModel(), index, true);
-			if(styleNode != null){
-				Color color = modeController.getExtension(EdgeController.class).getColor(styleNode);
+			final MapModel mapModel = map.getModel();
+			ModeController modeController = map.getModeController();
+			EdgeController edgeController = modeController.getExtension(EdgeController.class);
+			if(edgeController.areEdgeColorsAvailable(mapModel)){
+				Color color = edgeController.getEdgeColor(mapModel, index);
 				edgeColor.setCache(color);
 				return color;
 			}
@@ -1624,6 +1666,37 @@ public class NodeView extends JComponent implements INodeView {
 	boolean isHierarchyVisible() {
 		return getHeight() > 2 * getSpaceAround();
 	}
+	public enum Properties{HIDDEN_CHILDREN};
+	
+	boolean isChildHidden(NodeModel node) {
+		@SuppressWarnings("unchecked")
+		final Set<NodeModel> hiddenChildren = (Set<NodeModel>) getClientProperty(Properties.HIDDEN_CHILDREN);
+		return hiddenChildren != null && hiddenChildren.contains(node);
+	}
+
+	boolean hasHiddenChildren() {
+		@SuppressWarnings("unchecked")
+		final Set<NodeModel> hiddenChildren = (Set<NodeModel>) getClientProperty(Properties.HIDDEN_CHILDREN);
+		return hiddenChildren != null && ! hiddenChildren.isEmpty();
+	}
+
+	boolean unfoldHiddenChildren() {
+		final boolean hasHiddenChildren = hasHiddenChildren();
+		putClientProperty(Properties.HIDDEN_CHILDREN, null);
+		return hasHiddenChildren;
+	}
+
+	public void hideChildren(NodeModel node) {
+		final HashSet<NodeModel> set = new HashSet<>();
+		set.addAll(node.getChildren());
+		putClientProperty(Properties.HIDDEN_CHILDREN, set);
+	}
+
+	public boolean showHiddenNode(NodeModel node) {
+		@SuppressWarnings("unchecked")
+		final Set<NodeModel> hiddenChildren = (Set<NodeModel>) getClientProperty(Properties.HIDDEN_CHILDREN);
+		return hiddenChildren != null && hiddenChildren.remove(node);
+	}
 
 	
 }
